<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# ***** END LICENSE BLOCK *****

/**
 * \file
 * \brief Controls history viewing/diffing
 *
 * This file creates several hooks :
 * <ul>
 *   <li>history : to display list of available history</li>
 *   <li>history_view : to view a particular version by history id</li>
 *   <li>history_view_by_date : to view a particular version by date (the precision is the day)</li>
 *   <li>history_diff : to diff a version against another</li>
 *   <li>history_del : to delete a particular version. If the current version is deleted, then the latest version is restored as current content</li>
 * </ul>
 *
 * History viewing is done via a table. Each line is a particular version.
 * At the end of each line there is a radio button that allows the user to select 2 different versions.
 * <ul>
 * <li>If the user clicks on the version, or on the title, then the version is shown by history id.</li>
 * <li>If the user clicks on the date, then the version is shown by date (and the url used is a permalink).</li>
 * <li>If the user clicks on the "Show diff" button, then the two versions selected are diffed, which allows the user to see differences that happened between the versions</li>
 * </ul>
 *
 * This page contains also several helper functions for the user to create template :
 * <ul>
 *   <li>historyLink</li>
 * </ul>
 */

$contentBoxHandle["history"]              = "history";
$contentBoxHandle["history_view"]         = "historyView";
$contentBoxHandle["history_view_by_date"] = "historyViewByDate";
$contentBoxHandle["history_diff"]         = "historyDiff";

$userActionHandle["history_del"]          = "onDeleteHistory";
$userActionHandle["history_del_all"]      = "onDeleteAllHistory";

$browserTitlePageHandle["history"]              = "browserHistory";
$browserTitlePageHandle["history_view"]         = "browserHistoryView";
$browserTitlePageHandle["history_view_by_date"] = "browserHistoryViewByDate";
$browserTitlePageHandle["history_diff"]         = "browserHistoryDiff";

/**
 * \brief customize browser title when viewing history
 *
 * \return browser title
 */
function browserHistory()
{
  global $node;
  return wc_wiki_name.' - '.$node->title.' - '.tr("History");
}

/**
 * \brief customize browser title when viewing a version of the content
 *
 * \return browser title
 */
function browserHistoryView()
{
  global $node;

  if (!isset($_GET["id"]))
  {
    return;
  }

  $id = varGet("id");
  return wc_wiki_name.' - '.$node->title.' - '.tr("Version").' '.$id;
}

/**
 * \brief customize browser title when viewing history at a date
 *
 * \return browser title
 */
function browserHistoryViewByDate()
{
  global $node;

  if (!isset($_GET["year"]) || !isset($_GET["month"]) || !isset($_GET["day"]))
  {
    return;
  }

  $year  = varGet("year");
  $month = varGet("month");
  $day   = varGet("day");

  return wc_wiki_name.' - '.$node->title.' - '.tr("at").' '.$year.'/'.$month.'/'.$day;
}

/**
 * \brief customize browser title when diffing versions
 *
 * \return browser title
 */
function browserHistoryDiff()
{
  global $node;

  if (!isset($_POST["diff1"]) || !isset($_POST["diff2"]))
  {
    return;
  }

  $id1 = varPost("diff1");
  $id2 = varPost("diff2");
  if ($id2 == -1)
  {
    $id2 = "current";
  }

  return wc_wiki_name.' - '.$node->title.' - '.tr("Diff between version").' '.$id1.' '.tr("and version").' '.$id2;
}

require $path."/inc/classes/diff.class.php";

/**
 * \brief creates a link to history
 * \ingroup template_functions
 *
 * \return html link
 */
function historyLink()
{
  global $mode;

  if (!empty($mode))
  {
    return '';
  }

  global $node;

  if (empty($node))
  {
    return '';
  }

  return '<a href="'.createURL($node->title, array("mode"=>"history", "lang"=>$node->lang->code)).'">'.tr("View history").'</a>';
}

/**
 * \brief display history versions
 *
 * \return html blob
 */
function history()
{
  global $node;
  global $wiki;
  global $user;

  $canRead =  auth::canRead($user, $node);
  if ($canRead == false)
  {
    return '<p>'.tr("Sorry, you do not have enough rights to see this page").'</p>';
  }

  $history = array();
  $history[] = $node;
  $history = array_merge($history, $wiki->getHistory($node->id));

  $form = new form(createURL($node->title, array("mode"=>"history_diff", "lang"=>$node->lang->code)), "post", "diff");
  $form->addHtml("<p>");
  //sdw - add format of diff selection
  $form->addLabel(tr("Diff format"), "fformat");
  $form->addDropDown(array('Side by side comparision'=>'table', 
                            'Inline comparision' => 'inline',
                            'Unified comparision' => 'unified'), '' , "format", "fformat");
                            
  $form->addButton("diff", tr("Diff"), "submit");
  $form->addHtml("</p>");

  if ($user->isAdmin() || $user->isModerator())
  {
    $deleteAllUrl = '<a href="'.createUrl($node->title, array("mode"=>"history_del_all", "id"=>$node->id, "lang"=>$node->lang->code)).'">'.tr("Delete All").'</a>';
    $form->addHtml('<p>'.$deleteAllUrl.'</p>');
  }

  $form->addHtml('<table>');
  $form->addHtml('<tr>');

  if ($user->isAdmin() || $user->isModerator())
  {
    $form->addHtml('<th>'.tr("Delete").'</th>');
    $form->addHtml('<th>'.tr("Ban").'</th>');
  }

  $form->addHtml('<th>'.tr("Version").'</th>');
  $form->addHtml('<th>'.tr("Date").'</th>');
  $form->addHtml('<th>'.tr("Title").'</th>');
  $form->addHtml('<th>'.tr("Author").'</th>');
  $form->addHtml('<th>'.tr("Major/Minor").'</th>');
  $form->addHtml('<th>'.tr("Comment").'</th>');

  if ($user->isAdmin() || $user->isModerator())
  {
    $form->addHtml('<th>'.tr("Ip").'</th>');
  }

  $form->addHtml('<th></th>');
  $form->addHtml('<th></th>');
  $form->addHtml('</tr>');

  $even = true;
  $i = 0;
  $count = sizeof($history);

  foreach ($history as $n)
  {
    if ($i == 0)
    {
      $class = "first";
      $version = tr("Current");
    }
    else
    {
      $version = $count - $i;

      if ($even)
      {
        $class = "even";
      }
      else
      {
        $class = "uneven";
      }
    }

    $select1checked = false;
    $select2checked = false;
    $id = $n->id;
    if ($i == 0)
    {
      $id = -1;
      $url = createURL($node->title, array("lang"=>$node->lang->code));
      $select2checked = true;
    }
    else
    {
      if ($i == 1)
      {
        $select1checked = true;
      }
      $url = createURL($node->title, array("mode"=>"history_view", "id"=>$id, "lang"=>$node->lang->code));
    }

    $form->addHtml('<tr class="'.$class.'">');

    if ($user->isAdmin() || $user->isModerator())
    {
      if ($count > 1)
      {
        $deleteUrl = '<a href="'.createUrl($node->title, array("mode"=>"history_del", "id"=>$id, "lang"=>$node->lang->code)).'">'.tr("Delete").'</a>';
      }
      else
      {
        $deleteUrl = "";
      }

      $form->addHtml('<td>'.$deleteUrl.'</td>');
      $form->addHtml('<td><a href="'.createUrl($node->title, array("mode"=>"ban_add", "ip"=>$n->ip, "lang"=>$node->lang->code)).'">'.tr("Ban").'</a></td>');
    }

    $year   = date("Y", $n->timestamp);
    $month  = date("m", $n->timestamp);
    $day    = date("d", $n->timestamp);

    $urlByDate = createURL(
      $node->title,
      array(
        "mode"=>"history_view_by_date",
        "lang"=>$node->lang->code,
        "year"=>$year,
        "month"=>$month,
        "day"=>$day,
      )
    );

    $form->addHtml('<td><a href="'.$url.'">'.$version.'</a></td>');
    $form->addHtml('<td><a href="'.$urlByDate.'">'.formatDate($n->timestamp).'</a></td>');
    $form->addHtml('<td><a href="'.$url.'">'.$n->title.'</a></td>');
    $form->addHtml('<td>'.formatName($n->login, $n->first_name, $n->last_name).'</td>');
    $form->addHtml('<td>'.($n->minor_major==minor_value?tr("minor"):tr("major")).'</td>');
    $form->addHtml('<td>'.formatHtml($n->commit_msg).'</td>');

    if ($user->isAdmin() || $user->isModerator())
    {
      $form->addHtml('<td>'.$n->ip.'</td>');
    }

    $form->addHtml('<td>');
    if ($i != 0)
    {
      $form->addRadioButton("diff1", $n->id, $select1checked);
    }
    $form->addHtml('</td>');

    $form->addHtml('<td>');
    $form->addRadioButton("diff2", $id, $select2checked);
    $form->addHtml('</td>');

    $form->addHtml('</tr>');

    $even =  !$even;
    ++$i;
  }
  $form->addHtml('</table>');

  return $form->toHtml();
}

/**
 * \brief display a particular history version by id
 *
 * \return html blob
 */
function historyView()
{
  global $wiki;
  global $node;
  global $user;

  $canRead =  auth::canRead($user, $node);
  if ($canRead == false)
  {
    return '<p>'.tr("Sorry, you do not have enough rights to see this page").'</p>';
  }

  if (!isset($_GET["id"]))
  {
    return;
  }

  $id = varGet("id");

  $old_node = $wiki->getHistoryVersion($id);
  return $old_node->toHtml();
}

/**
 * \brief display a particular history version by date
 *
 * the precision is the date. The displayed version is the last version of the day.
 * It is possible to go beyond subsecond precision, but it will need to add more arguments
 * to url (at least hour, min, sec)
 *
 * \return html blob
 */
function historyViewByDate()
{
  global $wiki;
  global $node;
  global $user;

  $canRead =  auth::canRead($user, $node);
  if ($canRead == false)
  {
    return '<p>'.tr("Sorry, you do not have enough rights to see this page").'</p>';
  }

  if (!isset($_GET["year"]) || !isset($_GET["month"]) || !isset($_GET["day"]))
  {
    return;
  }

  $year  = varGet("year");
  $month = varGet("month");
  $day   = varGet("day");

  $timestamp = mktime(23, 59, 59, $month, $day, $year);

  $old_node = $wiki->getHistoryByDate($node->id, $timestamp);
  return $old_node->toHtml();
}

/**
 * \brief create title to display differences
 *
 * \param node the node to create title for
 * \return title
 */
function createTitle($node)
{
  return
  '<p>'.tr("Revision as of").' '.formatDate($node->timestamp).'</p>'.
  '<p>'.tr("By").' '.formatName($node->login, $node->first_name, $node->last_name).'</p>';
}

/**
 * \brief perform diff and display it
 *
 * \return html blob
 */
function historyDiff()
{
  global $wiki;
  global $node;
  global $user;

  $canRead =  auth::canRead($user, $node);
  if ($canRead == false)
  {
    return '<p>'.tr("Sorry, you do not have enough rights to see this page").'</p>';
  }

  if (!isset($_POST["diff1"]) || !isset($_POST["diff2"]))
  {
    return;
  }
  
  //steve - choose view type
  $type = isset($_POST['format']) ? $_POST['format'] : 'table';
    
  $id1 = varPost("diff1");
  $id2 = varPost("diff2");
  $html = '';

  $node1 = $wiki->getHistoryVersion($id1);
  $title1 = createTitle($node1);

  if ($id2 == -1)
  {
    $node2 = $node;
    $title2 =
    '<p>'.tr("Current version").'</p>'.
    '<p>'.tr("By").' '.formatName($node->login, $node->first_name, $node->last_name).'</p>';
  }
  else
  {
    $node2 = $wiki->getHistoryVersion($id2);
    $title2 = createTitle($node2);
  }

  $node1->content = $node1->content;
  $node2->content = $node2->content;
  
  $previous = explode("\n", formatHtml($node1->content));
  $current = explode("\n", formatHtml($node2->content));
  
  //steve - new diff process
  
  //include all required files from pear Text_Diff
  include( "Text_Diff/Diff.php" );
  include( "Text_Diff/Diff/Engine/native.php" );
  include( "Text_Diff/Diff/Renderer.php" );
  include( "Text_Diff/Diff/Renderer/inline.php" );
  include( "Text_Diff/Diff/Renderer/inline2.php" );
  include( "Text_Diff/Diff/Renderer/unified.php" );
  include( "Text_Diff/Diff/Renderer/table.php" );
  
  $diff = new Text_Diff( $previous, $current );
  
  /* Uncomment to view diff array
    echo '<pre>';
    print_r($diff);
    echo '</pre>';
  */
  
  if($type == 'inline'){
    $renderer = new Text_Diff_Renderer_inline2();
    $html .= '<h1>Diff - Inline camparision</h1>'."\n";
    $html .= '<div class="difftitle">'.$title2.'</div>'."\n";
    $html .= '<div class="difftitle">'.$title1.'</div>'."\n";        
    $html .= $renderer->render( $diff );
  }
   
  if($type == 'table'){
    $html .= '<h1>Diff - Side by side comparison </h1>'."\n";
    $renderer = new Text_Diff_Renderer_table();
    $html .= $renderer->render( $diff );
	//replace tiles in returned rendering of diff with the titles generated by wiclear
	$html = str_replace('<th>New</th>','<th>'.$title2.'</th>', $html);
	$html = str_replace('<th>Old</th>','<th>'.$title1.'</th>', $html);

  }
  
  if($type == 'unified'){
    $renderer = new Text_Diff_Renderer_unified();
    $html .= '<h1>Unified Diff</h1>'."\n";
    $html .= '<div class="difftitle">'.$title2.'</div>'."\n";
    $html .= '<div class="difftitle">'.$title1.'</div>'."\n";
    $html .= '<pre>';    
    $html .= $renderer->render( $diff );
    $html .= '</pre>';
  }
  return $html;
  
}

function onDeleteAllHistory()
{
  global $user;
  global $node;

  if ((!$user->isAdmin() && !$user->isModerator()) || !isset($_GET["id"]))
  {
    return;
  }

  global $wiki;

  $id = varGet("id");

  // button delete all history has been used
  $wiki->deleteAllHistory($id);
  header('Location: '.createHeaderURL($node->title, array("mode"=>"history", "lang"=>$node->lang->code)));
}

/**
 * \brief delete history from history form
 */
function onDeleteHistory()
{
  global $user;
  global $node;

  if ((!$user->isAdmin() && !$user->isModerator()) || !isset($_GET["id"]))
  {
    return;
  }

  global $wiki;

  $id = varGet("id");

  if ($id == -1)
  {
    // this means we want to restore last version and delete current one
    $history = $wiki->getHistory($node->id);
    if (sizeof($history) > 0)
    {
      $lastVersion = $history[0];

      $fake_user = new user(
        $lastVersion->user_id,
        $lastVersion->login,
        "", /*password*/
        user_value,
        $lastVersion->first_name,
        $lastVersion->last_name,
        $lastVersion->email,
        $lastVersion->lang->id,
        $lastVersion->lang->id,
        time()
      );

      $wiki->editContent(
        $node->id, $node->parent_id, $lastVersion->parent_id,
        $lastVersion->title, $lastVersion->alternate_title, $lastVersion->content,
        $lastVersion->commit_msg, $lastVersion->minor_major,
        $node->related_content_id,
        $fake_user,
        $node->left, $node->right
      );

      $id = $lastVersion->id;
    }
    else
    {
      $id = -1;
    }
  }

  $wiki->deleteHistory($id);

  header('Location: '.createHeaderURL($node->title, array("mode"=>"history", "lang"=>$node->lang->code)));
}
?>
